package main

// NOTE: This is copied over from tendermint/tendermint/libs/common/colors.go
// Do not edit this file without modifying the other.

import (
	"fmt"
	"strings"
)

const (
	ANSIReset = "\x1b[0m"

	ANSIFgRed    = "\x1b[31m"
	ANSIFgGreen  = "\x1b[32m"
	ANSIFgYellow = "\x1b[33m"
	ANSIFgBlue   = "\x1b[34m"
	ANSIFgCyan   = "\x1b[36m"
)

// color the string s with color 'color'
// unless s is already colored
func treat(s string, color string) string {
	if len(s) > 2 && s[:2] == "\x1b[" {
		return s
	}
	return color + s + ANSIReset
}

func treatAll(color string, args ...interface{}) string {
	var parts = make([]string, len(args))
	for _, arg := range args {
		parts = append(parts, treat(fmt.Sprintf("%v", arg), color))
	}
	return strings.Join(parts, "")
}

func Red(args ...interface{}) string {
	return treatAll(ANSIFgRed, args...)
}

func Green(args ...interface{}) string {
	return treatAll(ANSIFgGreen, args...)
}

func Yellow(args ...interface{}) string {
	return treatAll(ANSIFgYellow, args...)
}

func Blue(args ...interface{}) string {
	return treatAll(ANSIFgBlue, args...)
}

func Cyan(args ...interface{}) string {
	return treatAll(ANSIFgCyan, args...)
}

func ColoredBytes(data []byte, textColor, bytesColor func(...interface{}) string) string {
	s := ""
	for _, b := range data {
		if 0x21 <= b && b < 0x7F {
			s += textColor(string(b))
		} else {
			s += bytesColor(fmt.Sprintf("%02X", b))
		}
	}
	return s
}
